@@ -1,61 +1,98 @@
-.makehalves <- function(samples, splits, random) {
-	if (!missing(splits) && missing(random)) {
-		# should be even because we want to combine into halves
-		# should also have at least as many samples as splits
-		stopifnot(splits %% 2 == 0, length(samples) >= splits)
-
-		# a list of length(.) == splits
-		groups <- split(samples, rep_len(1:splits, length(samples)))
-
-		# use combn() to generate combinations of groups[[i]]
-		# taken splits/2 at at time i.e. build halves from them
-		unique(combn(
-			groups, splits / 2, function(cmb) {
-				# not sure where the names are coming from, but they
-				# break the tests for identity in unique()
-				left <- unname(sort(unlist(cmb)))
-				# generate the other half right away (AB => CD)
-				right <- unname(sort(setdiff(samples, left)))
-				# this approach would give us both pairs AB vs CD and CD vs AB
-				# so we sort both halves, place the half with the
-				# lowest-numbered sample first, then eliminate duplicates
-				list(left, right)[order(c(min(left), min(right)))]
-			}, FALSE
-		))
-	} else if (!missing(random) && missing(splits)) {
-		# should have at least 2 samples in order to separate into halves
-		stopifnot(length(samples) >= 2)
-		half <- floor(length(samples) / 2)
-		replicate(random, {
-			samples <- sample(samples)
-			list(samples[1:half], samples[-(1:half)])
-		}, FALSE)
-	} else stop(
-		"Please provide either splits = n.groups or random = n.shuffles"
+.splitcombine <- function(samples, splits) {
+	# splits must be a scalar or a 2-element vector of whole numbers
+	# splits[1] should be even because we want to combine into halves
+	# we should also have at least as many samples as splits[1]
+	stopifnot(
+		length(splits) <= 2, splits == round(splits),
+		splits[1] %% 2 == 0,
+		length(samples) >= splits[1],
+		is.na(splits[2]) || splits[2] <= choose(splits[1], splits[1]/2)/2
 	)
+	if (is.na(splits[2])) splits[2] <- choose(splits[1], splits[1]/2)/2
+
+	# we now have `splits` groups to recombine into halves
+	groups <- split(samples, rep_len(1:splits[1], length(samples)))
+
+	# use combn() to generate combinations of groups[[i]]
+	# taken splits[1]/2 at at time i.e. build halves from them
+	unique(combn(
+		groups, splits[1] / 2, function(cmb) {
+			# not sure where the names are coming from, but they
+			# break the tests for identity in unique()
+			left <- unname(sort(unlist(cmb)))
+			# generate the other half right away (AB => CD)
+			right <- unname(sort(setdiff(samples, left)))
+			# this approach would give us both pairs AB vs CD and CD vs AB
+			# so we sort both halves, place the half with the
+			# lowest-numbered sample first, then eliminate duplicates
+			list(left, right)[order(c(min(left), min(right)))]
+		}, FALSE
+	))[1:splits[2]]
+}
+
+.randomhalves <- function(samples, N) {
+	# should have at least 2 samples in order to separate into halves
+	stopifnot(length(samples) >= 2)
+
+	half <- floor(length(samples) / 2)
+	replicate(N, {
+		samples <- sample(samples)
+		list(samples[1:half], samples[-(1:half)])
+	}, FALSE)
 }
 
 feemsplithalf <- function(
-	cube, nfac, splits, random, groups, ..., progress = TRUE
+	cube, nfac, splits, random, groups, fixed, ..., progress = TRUE
 ) {
-	# if not performing stratified sampling, create one fake group
-	# encasing all samples
-	if (missing(groups)) groups <- rep_len(1, dim(cube)[3])
-	stopifnot(
-		(if (is.list(groups)) lengths else length)(groups) == dim(cube)[3]
-	)
-	# list(list(half, half), ...)
-	tests <- Reduce(
-		# given same-sized per-group lists of halves,
-		function(a, b) lapply(
-			# concatenate individual halves between groups
-			seq_along(a), function(i) Map(c, a[[i]], b[[i]])
-		),
-		lapply( # list(group = list(list(half, half), ...), ...)
-			split(1:dim(cube)[3], groups, drop = TRUE),
-			.makehalves,
-			splits = splits, random = random
+	tests <- if (
+		!missing(fixed) &&
+		missing(splits) && missing(random) && missing(groups)
+	) {
+		# must be a list of pairs
+		# pair is a list of two vectors containing integer indices
+		stopifnot(
+			is.list(fixed), lengths(fixed) == 2,
+			unlist(fixed) == round(unlist(fixed))
 		)
+		for (i in seq_along(fixed)) {
+			if (length(s <- intersect(
+				fixed[[i]][[1]], fixed[[i]][[2]]
+			)) > 0)
+				stop(
+					'Both halves in fixed[[', i, ']] contain the ',
+					'following samples: ', paste(s, collapse = ', ')
+				)
+		}
+		fixed
+	} else if (
+		missing(splits) != missing(random) && # exactly one of them set
+		missing(fixed)
+	) {
+		# if not performing stratified sampling, create one fake group
+		# encasing all samples
+		if (missing(groups)) groups <- rep_len(1, dim(cube)[3])
+		stopifnot(
+			(if (is.list(groups)) lengths else length)(groups) == dim(cube)[3]
+		)
+		# list(list(half, half), ...)
+		Reduce(
+			# given same-sized per-group lists of halves,
+			function(a, b) lapply(
+				# concatenate individual halves between groups
+				seq_along(a), function(i) Map(c, a[[i]], b[[i]])
+			),
+			lapply( # list(group = list(list(half, half), ...), ...)
+				split(1:dim(cube)[3], groups, drop = TRUE),
+				if (!missing(splits))
+					function(s) .splitcombine(s, splits)
+				else if (!missing(random))
+					function(s) .randomhalves(s, random)
+			)
+		)
+	} else stop(
+		'Please either request split-combine or random halves ',
+		'(optionally stratified by groups), ',
+		'or provide fixed halves.'
 	)
 	# organise two parallel arrays:
 	# ((half1, half2), (half3, half4), ...) =>
@@ -98,10 +135,10 @@ feemsplithalf <- function(
 				1:dim(factors)[3], function(grp) # for each grouping,
 					vapply( # for each mode,
 						c('A','B'), function(mode) # TCC for this half and mode
-							diag(congru(
+							diag(as.matrix(congru(
 								factors[[1,ifac,grp]][[mode]],
 								factors[[2,ifac,grp]][[mode]]
-							)),
+							))),
 							# only for matching components
 						numeric(nfac[ifac])
 					),
@@ -200,3 +237,9 @@ coef.feemsplithalf <- function(object, kind = c('tcc', 'factors'), ...) {
 		tcc = coefshtcc(object)
 	)
 }
+
+feemcube.feemsplithalf <- function(x, ...) {
+	stopifnot(length(list(...)) == 0)
+	x <- x$factors[[1]]
+	get(attr(x, 'cube'), envir = attr(x, 'envir'))
+}
